package ru.dbotthepony.kbox2d.api

import kotlin.math.PI

/// You can use this to change the length scale used by your game.
/// For example for inches you could use 39.4.
const val b2_lengthUnitsPerMeter = 1.0

/// The maximum number of vertices on a convex polygon. You cannot increase
/// this too much because b2BlockAllocator has a maximum object size.
const val b2_maxPolygonVertices = 8

/// Collision

/// The maximum number of contact points between two convex shapes. Do
/// not change this value.
const val b2_maxManifoldPoints = 2

/// This is used to fatten AABBs in the dynamic tree. This allows proxies
/// to move by a small amount without triggering a tree adjustment.
/// This is in meters.
const val b2_aabbExtension = b2_lengthUnitsPerMeter * 0.1

/// This is used to fatten AABBs in the dynamic tree. This is used to predict
/// the future position based on the current displacement.
/// This is a dimensionless multiplier.
const val b2_aabbMultiplier = 4.0

/// A small length used as a collision and constraint tolerance. Usually it is
/// chosen to be numerically significant, but visually insignificant. In meters.
const val b2_linearSlop = 0.005 * b2_lengthUnitsPerMeter

/// A small angle used as a collision and constraint tolerance. Usually it is
/// chosen to be numerically significant, but visually insignificant.
const val b2_angularSlop = 2.0 / 180.0 * PI

/// The radius of the polygon/edge shape skin. This should not be modified. Making
/// this smaller means polygons will have an insufficient buffer for continuous collision.
/// Making it larger may create artifacts for vertex collision.
const val b2_polygonRadius = 2.0 * b2_linearSlop

/// Maximum number of sub-steps per contact in continuous physics simulation.
const val b2_maxSubSteps = 8

// Dynamics

/// Maximum number of contacts to be handled to solve a TOI impact.
const val b2_maxTOIContacts = 32

/// The maximum linear position correction used when solving constraints. This helps to
/// prevent overshoot. Meters.
const val b2_maxLinearCorrection = (0.2 * b2_lengthUnitsPerMeter)

/// The maximum angular position correction used when solving constraints. This helps to
/// prevent overshoot.
const val b2_maxAngularCorrection = (8.0 / 180.0 * PI)

/// The maximum linear translation of a body per step. This limit is very large and is used
/// to prevent numerical problems. You shouldn't need to adjust this. Meters.
const val b2_maxTranslation = (2.0 * b2_lengthUnitsPerMeter)
const val b2_maxTranslationSquared = (b2_maxTranslation * b2_maxTranslation)

/// The maximum angular velocity of a body. This limit is very large and is used
/// to prevent numerical problems. You shouldn't need to adjust this.
const val b2_maxRotation = (0.5 * PI)
const val b2_maxRotationSquared = (b2_maxRotation * b2_maxRotation)

/// This scale factor controls how fast overlap is resolved. Ideally this would be 1 so
/// that overlap is removed in one time step. However using values close to 1 often lead
/// to overshoot.
const val b2_baumgarte = 0.2
const val b2_toiBaumgarte = 0.75

// Sleep

/// The time that a body must be still before it will go to sleep.
const val b2_timeToSleep = 0.5

/// A body cannot sleep if its linear velocity is above this tolerance.
const val b2_linearSleepTolerance = (0.01 * b2_lengthUnitsPerMeter)

/// A body cannot sleep if its angular velocity is above this tolerance.
const val b2_angularSleepTolerance = (2.0f / 180.0f * PI)

const val b2_epsilon = 1E-9
