package ru.dbotthepony.kvector.vector

import ru.dbotthepony.kvector.api.IStruct4f
import ru.dbotthepony.kvector.vector.nfloat.Vector4f
import java.util.Collections

/**
 * Represents RGBA color in [0, 1] range using [Float].
 */
class Color(r: Float, g: Float, b: Float, a: Float = 1f) : Vector4f(r, g, b, a) {
	constructor(input: IStruct4f) : this(input.component1(), input.component2(), input.component3(), input.component4())

	constructor(input: Int) : this(
		((input ushr 16) and 0xFF).toFloat() / 255f,
		((input ushr 8) and 0xFF).toFloat() / 255f,
		(input and 0xFF).toFloat() / 255f,
	)

	fun copy(r: Float = this.r, g: Float = this.g, b: Float = this.b, a: Float = this.a): Color {
		return Color(r, g, b, a)
	}

	companion object {
		val WHITE = Color(1f, 1f, 1f)

		val RED = Color(1f, 0f, 0f)
		val GREEN = Color(0f, 1f, 0f)
		val BLUE = Color(0f, 0f, 1f)

		val SLATE_GREY = Color(0.2f, 0.2f, 0.2f)

		val PRE_DEFINED_MAP = mapOf(
			"red" to RED,
			"green" to GREEN,
			"blue" to BLUE,
		)

		val SHADES_OF_GRAY = ArrayList<Color>().let {
			for (i in 0 .. 256) {
				it.add(Color(i / 256f, i / 256f, i / 256f))
			}

			return@let Collections.unmodifiableList(it) as List<Color>
		}
	}
}
